/* Generated by re2c 3.1 on Tue Sep 17 20:28:32 2024 */
#line 1 "../src/main.ypp"
// Copyright (c) 2020, 2021 Vinícius dos Santos Oliveira
// SPDX-License-Identifier: MIT OR BSL-1.0

#include <string_view>
#include <charconv>
#include <optional>

#include <fmt/ostream.h>
#include <fmt/format.h>

#include <boost/preprocessor/stringize.hpp>
#include <boost/nowide/iostream.hpp>
#include <boost/nowide/args.hpp>
#include <boost/version.hpp>

#include <emilua/windows.hpp>
#include <emilua/state.hpp>

#if EMILUA_CONFIG_USE_STANDALONE_ASIO
#include <asio/io_context.hpp>
#else // EMILUA_CONFIG_USE_STANDALONE_ASIO
#include <boost/asio/io_context.hpp>
#endif // EMILUA_CONFIG_USE_STANDALONE_ASIO

#if BOOST_OS_LINUX
#include <boost/context/fixedsize_stack.hpp>
#endif // BOOST_OS_LINUX

#if EMILUA_CONFIG_USE_STANDALONE_ASIO
#include <asio/version.hpp>
#endif // EMILUA_CONFIG_USE_STANDALONE_ASIO

#if EMILUA_CONFIG_ENABLE_COLOR
#include <cstdlib>
#include <cstdio>

extern "C" {
#include <curses.h>
#include <term.h>
} // extern "C"
#endif // EMILUA_CONFIG_ENABLE_COLOR

#if BOOST_OS_UNIX
#include <emilua/actor.hpp>
#include <sys/wait.h>
#endif // BOOST_OS_UNIX

namespace hana = boost::hana;
namespace fs = std::filesystem;

#if EMILUA_CONFIG_USE_STANDALONE_ASIO
namespace asio = ::asio;
#else // EMILUA_CONFIG_USE_STANDALONE_ASIO
namespace asio = boost::asio;
#endif // EMILUA_CONFIG_USE_STANDALONE_ASIO

extern char** environ;

static constexpr auto help_text = FMT_STRING(
    "Emilua: Execution engine for luaJIT\n"
    "Usage: {} [OPTIONS] [file]\n"
    "\n"
    "Positionals:\n"
    "  file TEXT                   Script filename\n"
    "\n"
    "Options:\n"
    "  -h,--help                   Print this help message and exit\n"
    "  --main-context-concurrency-hint INT\n"
    "                              Concurrency hint for the main execution engine context\n"
    "  --test Run tests\n"
    "  --version                   Output version information and exit\n");

int main(int argc, char *argv[], char *envp[])
{
#if BOOST_OS_UNIX
    {
        struct sigaction sa;
        sa.sa_handler = SIG_DFL;
        sigemptyset(&sa.sa_mask);
        sa.sa_flags = 0;

        for (int signo = 1 ; signo != NSIG ; ++signo) {
            sigaction(signo, /*act=*/&sa, /*oldact=*/NULL);
        }

        sigset_t set;
        sigfillset(&set);
        sigprocmask(SIG_UNBLOCK, &set, /*oldset=*/NULL);
    }
#endif // BOOST_OS_UNIX

#if BOOST_OS_LINUX
    boost::context::fixedsize_stack clone_stack_allocator;
    auto clone_stack = clone_stack_allocator.allocate();
    emilua::clone_stack_address = clone_stack.sp;
#endif // BOOST_OS_LINUX

#if BOOST_OS_UNIX
    emilua::app_context::environp = &environ;

    int ipc_actor_service_pipe[2];
    if (socketpair(AF_UNIX, SOCK_SEQPACKET, 0, ipc_actor_service_pipe) == -1) {
        ipc_actor_service_pipe[0] = -1;
        ipc_actor_service_pipe[1] = -1;
        perror("<4>Failed to start subprocess-based actor subsystem");
    }

    if (ipc_actor_service_pipe[0] != -1) {
        shutdown(ipc_actor_service_pipe[0], SHUT_WR);
        shutdown(ipc_actor_service_pipe[1], SHUT_RD);

        switch (fork()) {
        case -1: {
            perror("<4>Failed to start subprocess-based actor subsystem");
            close(ipc_actor_service_pipe[0]);
            close(ipc_actor_service_pipe[1]);
            ipc_actor_service_pipe[0] = -1;
            ipc_actor_service_pipe[1] = -1;
            break;
        }
        case 0:
            close(ipc_actor_service_pipe[1]);
            return emilua::app_context::ipc_actor_service_main(
                ipc_actor_service_pipe[0]);
        default: {
            close(ipc_actor_service_pipe[0]);
            ipc_actor_service_pipe[0] = -1;

            emilua::bzero_region args;

            // These regions are computed from this process so we try (as the
            // best of our efforts) to avoid loading them into extra
            // registers/stack of the other process.
            //
            // Ideally we'd control the whole chain of executed instructions up
            // to the point of calling explicit_bzero(), but that means writing
            // our own libc implementation and writing everything in
            // assembly. It's not really feasible to do the right thing here so
            // we just settle for a weaker compromise using portable C + glibc's
            // explicit_bzero().
            //
            // The alternative execve()-based approach provides stronger
            // isolation by flushing the whole address space, but comes with its
            // own limitations. By providing both approaches we can attack a
            // wider range of problems. IOW, there are no plans to remove this
            // code (the bzero() protection might not be a complete solution,
            // but it works usually well in practice).
            for (int i = 1 ; i < argc ; ++i) {
                std::string_view view = argv[i];
                args.s = const_cast<char*>(view.data());
                args.n = view.size();
                write(ipc_actor_service_pipe[1], &args, sizeof(args));
            }
            for (char** rawenv = envp ; *rawenv ; ++rawenv) {
                std::string_view env{*rawenv};
                args.s = const_cast<char*>(env.data());
                args.n = env.size();
                write(ipc_actor_service_pipe[1], &args, sizeof(args));
            }
            args.s = nullptr;
            args.n = 0;
            write(ipc_actor_service_pipe[1], &args, sizeof(args));
        }
        }
    }

    if (EMILUA_CONFIG_EINTR_RTSIGNO != 0) {
        struct sigaction sa;
        std::memset(&sa, 0, sizeof(struct sigaction));

        // from SIG_UNBLOCK to SIG_BLOCK, execution MUST only use
        // async-signal-safe code
        sigemptyset(&sa.sa_mask);
        sigaddset(&sa.sa_mask, EMILUA_CONFIG_EINTR_RTSIGNO);
        sigprocmask(SIG_BLOCK, &sa.sa_mask, /*oldset=*/NULL);

        sa.sa_sigaction = emilua::longjmp_on_rtsigno;
        sa.sa_flags = SA_RESTART | SA_SIGINFO;
        sigaction(EMILUA_CONFIG_EINTR_RTSIGNO, /*act=*/&sa, /*oldact=*/NULL);
    }
#endif // BOOST_OS_UNIX

    try {
        std::locale native_locale{""};
        std::locale::global(native_locale);
        boost::nowide::cin.imbue(native_locale);
        boost::nowide::cout.imbue(native_locale);
        boost::nowide::cerr.imbue(native_locale);
        boost::nowide::clog.imbue(native_locale);
    } catch (const std::exception& e) {
        try {
            fmt::print(
                boost::nowide::cerr,
                FMT_STRING("<4>Failed to set the native locale: `{}`\n"),
                e.what());
        } catch (const std::ios_base::failure&) {}
    }
    std::optional<boost::nowide::args> args = std::nullopt;
    try {
        args.emplace(argc, argv, envp);
    } catch (const std::exception& e) {
        try {
            fmt::print(
                boost::nowide::cerr,
                FMT_STRING("Failed to read args/envp: `{}`\n"),
                e.what());
        } catch (const std::ios_base::failure&) {}
        return 2;
    }

    std::unordered_map<std::string_view, std::string_view> tmp_env;
    for (char** rawenv = envp ; *rawenv ; ++rawenv) {
        std::string_view env{*rawenv};
        auto i = env.find('=');
        auto key = env.substr(0, i);
        auto value = env.substr(i + 1);
        tmp_env.emplace(key, value);
    }

#if EMILUA_CONFIG_ENABLE_COLOR
    emilua::stdout_has_color = [&tmp_env]() {
        if (auto it = tmp_env.find("EMILUA_COLORS") ; it != tmp_env.end()) {
            if (it->second == "1") {
                return true;
            } else if (it->second == "0") {
                return false;
            } else if (it->second.size() > 0) {
                try {
                    boost::nowide::cerr <<
                        "<4>Ignoring unrecognized value for EMILUA_COLORS\n";
                } catch (const std::ios_base::failure&) {}
            }
        }

        // Emilua runtime by itself will only ever dirt stderr
        if (!isatty(fileno(stderr)))
            return false;

        int ec = 0;
        if (setupterm(NULL, fileno(stderr), &ec) == ERR)
            return false;

        bool ret = tigetnum("colors") > 0;
        del_curterm(cur_term);
        return ret;
    }();
#else
    emilua::stdout_has_color = false;
#endif // EMILUA_CONFIG_ENABLE_COLOR

    if (auto it = tmp_env.find("EMILUA_LOG_LEVELS") ; it != tmp_env.end()) {
        std::string_view env = it->second;
        int level;
        auto res = std::from_chars(
            env.data(), env.data() + env.size(), level);
        if (res.ec == std::errc{})
            emilua::log_domain<emilua::default_log_domain>::log_level = level;
    }

    std::string_view filename;
#if EMILUA_CONFIG_USE_STANDALONE_ASIO
    int main_ctx_concurrency_hint = ASIO_CONCURRENCY_HINT_SAFE;
#else // EMILUA_CONFIG_USE_STANDALONE_ASIO
    int main_ctx_concurrency_hint = BOOST_ASIO_CONCURRENCY_HINT_SAFE;
#endif // EMILUA_CONFIG_USE_STANDALONE_ASIO
    emilua::ContextType main_context_type = emilua::ContextType::main;
    emilua::app_context appctx;
    appctx.app_env = std::move(tmp_env);

#if BOOST_OS_UNIX
    appctx.ipc_actor_service_sockfd = ipc_actor_service_pipe[1];
#endif // BOOST_OS_UNIX

#define NEXT_ARG(OPTION, LABEL) do {                           \
        if (YYCURSOR = *++cur_arg ; YYCURSOR != nullptr) {     \
            goto LABEL;                                        \
        } else {                                               \
            try {                                              \
                boost::nowide::cerr <<                         \
                    "missing value for option `" OPTION "`\n"; \
            } catch (const std::ios_base::failure&) {}         \
            return 2;                                          \
        }                                                      \
    } while(0)

#define ERRARG(OPTION)                                                     \
    do {                                                                   \
        try {                                                              \
            fmt::print(                                                    \
                boost::nowide::cerr,                                       \
                FMT_STRING(                                                \
                    "value `{}` not supported for option `" OPTION "`\n"), \
                *cur_arg);                                                 \
        } catch (const std::ios_base::failure&) {}                         \
        return 2;                                                          \
    } while(0)

    #line 305 "../src/main.ypp"

    char* YYCURSOR;
    char* YYMARKER;
    char** cur_arg = argv;
opt:
    YYCURSOR = *++cur_arg;
    if (YYCURSOR == nullptr) goto end;

#line 310 "emilua.p/main.cpp"
{
    char yych;
    yych = *YYCURSOR;
    switch (yych) {
        case 0x00: goto yy1;
        case '-': goto yy3;
        default: goto yy2;
    }
yy1:
    ++YYCURSOR;
#line 313 "../src/main.ypp"
    {
        try {
            fmt::print(boost::nowide::cerr, FMT_STRING("bad option: `{}`\n"),
                       *cur_arg);
        } catch (const std::ios_base::failure&) {}
        return 2;
    }
#line 329 "emilua.p/main.cpp"
yy2:
    yych = *++YYCURSOR;
    if (yych <= 0x00) goto yy5;
    goto yy2;
yy3:
    yych = *++YYCURSOR;
    switch (yych) {
        case 0x00: goto yy6;
        case '-': goto yy7;
        default: goto yy4;
    }
yy4:
#line 346 "../src/main.ypp"
    { goto opt_short; }
#line 344 "emilua.p/main.cpp"
yy5:
    ++YYCURSOR;
#line 335 "../src/main.ypp"
    {
        if (filename.size() > 0) {
            try {
                boost::nowide::cerr <<
                    "emilua doesn't support more than one input file\n";
            } catch (const std::ios_base::failure&) {}
            return 2;
        }
        filename = *cur_arg;
        goto opt;
    }
#line 359 "emilua.p/main.cpp"
yy6:
    ++YYCURSOR;
#line 329 "../src/main.ypp"
    {
        try {
            boost::nowide::cerr << "stdin not supported as input file\n";
        } catch (const std::ios_base::failure&) {}
        return 2;
    }
#line 369 "emilua.p/main.cpp"
yy7:
    yych = *++YYCURSOR;
    if (yych <= 0x00) goto yy8;
#line 347 "../src/main.ypp"
    { goto opt_long; }
#line 375 "emilua.p/main.cpp"
yy8:
    ++YYCURSOR;
#line 320 "../src/main.ypp"
    {
        appctx.app_args.reserve(2 + ((argv + argc) - (cur_arg + 1)));
        appctx.app_args.emplace_back(argv[0]);
        appctx.app_args.emplace_back(filename);
        while (*++cur_arg) {
            appctx.app_args.emplace_back(*cur_arg);
        }
        goto end;
    }
#line 388 "emilua.p/main.cpp"
}
#line 348 "../src/main.ypp"


opt_short:

#line 395 "emilua.p/main.cpp"
{
    char yych;
    yych = *YYCURSOR;
    switch (yych) {
        case 0x00: goto yy10;
        case 'h': goto yy12;
        default: goto yy11;
    }
yy10:
    ++YYCURSOR;
#line 359 "../src/main.ypp"
    { goto opt; }
#line 408 "emilua.p/main.cpp"
yy11:
    ++YYCURSOR;
#line 352 "../src/main.ypp"
    {
        try {
            fmt::print(boost::nowide::cerr, FMT_STRING("bad short option: {}\n"),
                       *cur_arg);
        } catch (const std::ios_base::failure&) {}
        return 2;
    }
#line 419 "emilua.p/main.cpp"
yy12:
    ++YYCURSOR;
#line 360 "../src/main.ypp"
    {
        try {
            fmt::print(boost::nowide::cout, help_text, *argv);
        } catch (const std::ios_base::failure&) {}
        return 0;
    }
#line 429 "emilua.p/main.cpp"
}
#line 366 "../src/main.ypp"


opt_long:

#line 436 "emilua.p/main.cpp"
{
    char yych;
    yych = *YYCURSOR;
    switch (yych) {
        case 'h': goto yy16;
        case 'm': goto yy17;
        case 't': goto yy18;
        case 'v': goto yy19;
        default: goto yy14;
    }
yy14:
    ++YYCURSOR;
yy15:
#line 370 "../src/main.ypp"
    {
        try {
            fmt::print(boost::nowide::cerr, FMT_STRING("bad long option: {}\n"),
                       *cur_arg);
        } catch (const std::ios_base::failure&) {}
        return 2;
    }
#line 458 "emilua.p/main.cpp"
yy16:
    yych = *(YYMARKER = ++YYCURSOR);
    switch (yych) {
        case 'e': goto yy20;
        default: goto yy15;
    }
yy17:
    yych = *(YYMARKER = ++YYCURSOR);
    switch (yych) {
        case 'a': goto yy22;
        default: goto yy15;
    }
yy18:
    yych = *(YYMARKER = ++YYCURSOR);
    switch (yych) {
        case 'e': goto yy23;
        default: goto yy15;
    }
yy19:
    yych = *(YYMARKER = ++YYCURSOR);
    switch (yych) {
        case 'e': goto yy24;
        default: goto yy15;
    }
yy20:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'l': goto yy25;
        default: goto yy21;
    }
yy21:
    YYCURSOR = YYMARKER;
    goto yy15;
yy22:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'i': goto yy26;
        default: goto yy21;
    }
yy23:
    yych = *++YYCURSOR;
    switch (yych) {
        case 's': goto yy27;
        default: goto yy21;
    }
yy24:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'r': goto yy28;
        default: goto yy21;
    }
yy25:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'p': goto yy29;
        default: goto yy21;
    }
yy26:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'n': goto yy30;
        default: goto yy21;
    }
yy27:
    yych = *++YYCURSOR;
    switch (yych) {
        case 't': goto yy31;
        default: goto yy21;
    }
yy28:
    yych = *++YYCURSOR;
    switch (yych) {
        case 's': goto yy32;
        default: goto yy21;
    }
yy29:
    yych = *++YYCURSOR;
    if (yych <= 0x00) goto yy33;
    goto yy21;
yy30:
    yych = *++YYCURSOR;
    switch (yych) {
        case '-': goto yy34;
        default: goto yy21;
    }
yy31:
    yych = *++YYCURSOR;
    if (yych <= 0x00) goto yy35;
    goto yy21;
yy32:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'i': goto yy36;
        default: goto yy21;
    }
yy33:
    ++YYCURSOR;
#line 377 "../src/main.ypp"
    {
        try {
            fmt::print(boost::nowide::cout, help_text, *argv);
        } catch (const std::ios_base::failure&) {}
        return 0;
    }
#line 563 "emilua.p/main.cpp"
yy34:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'c': goto yy37;
        default: goto yy21;
    }
yy35:
    ++YYCURSOR;
#line 407 "../src/main.ypp"
    {
        main_context_type = emilua::ContextType::test;
        goto opt;
    }
#line 577 "emilua.p/main.cpp"
yy36:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'o': goto yy38;
        default: goto yy21;
    }
yy37:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'o': goto yy39;
        default: goto yy21;
    }
yy38:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'n': goto yy40;
        default: goto yy21;
    }
yy39:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'n': goto yy41;
        default: goto yy21;
    }
yy40:
    yych = *++YYCURSOR;
    if (yych <= 0x00) goto yy42;
    goto yy21;
yy41:
    yych = *++YYCURSOR;
    switch (yych) {
        case 't': goto yy43;
        default: goto yy21;
    }
yy42:
    ++YYCURSOR;
#line 383 "../src/main.ypp"
    {
        try {
            boost::nowide::cout << "Emilua " EMILUA_CONFIG_VERSION_STRING <<
                std::endl << LUAJIT_VERSION << std::endl <<
                "Boost " << (BOOST_VERSION / 100000) << '.' <<
                (BOOST_VERSION / 100 % 1000) << '.' << (BOOST_VERSION % 100) <<
                std::endl <<
#if EMILUA_CONFIG_USE_STANDALONE_ASIO
                "ASIO " << (ASIO_VERSION / 100000) << '.' <<
                (ASIO_VERSION / 100 % 1000) << '.' << (ASIO_VERSION % 100) <<
                std::endl <<
#endif // EMILUA_CONFIG_USE_STANDALONE_ASIO
                std::flush;
        } catch (const std::ios_base::failure&) {}
        return 0;
    }
#line 631 "emilua.p/main.cpp"
yy43:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'e': goto yy44;
        default: goto yy21;
    }
yy44:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'x': goto yy45;
        default: goto yy21;
    }
yy45:
    yych = *++YYCURSOR;
    switch (yych) {
        case 't': goto yy46;
        default: goto yy21;
    }
yy46:
    yych = *++YYCURSOR;
    switch (yych) {
        case '-': goto yy47;
        default: goto yy21;
    }
yy47:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'c': goto yy48;
        default: goto yy21;
    }
yy48:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'o': goto yy49;
        default: goto yy21;
    }
yy49:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'n': goto yy50;
        default: goto yy21;
    }
yy50:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'c': goto yy51;
        default: goto yy21;
    }
yy51:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'u': goto yy52;
        default: goto yy21;
    }
yy52:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'r': goto yy53;
        default: goto yy21;
    }
yy53:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'r': goto yy54;
        default: goto yy21;
    }
yy54:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'e': goto yy55;
        default: goto yy21;
    }
yy55:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'n': goto yy56;
        default: goto yy21;
    }
yy56:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'c': goto yy57;
        default: goto yy21;
    }
yy57:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'y': goto yy58;
        default: goto yy21;
    }
yy58:
    yych = *++YYCURSOR;
    switch (yych) {
        case '-': goto yy59;
        default: goto yy21;
    }
yy59:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'h': goto yy60;
        default: goto yy21;
    }
yy60:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'i': goto yy61;
        default: goto yy21;
    }
yy61:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'n': goto yy62;
        default: goto yy21;
    }
yy62:
    yych = *++YYCURSOR;
    switch (yych) {
        case 't': goto yy63;
        default: goto yy21;
    }
yy63:
    yych = *++YYCURSOR;
    switch (yych) {
        case 0x00: goto yy64;
        case '=': goto yy65;
        default: goto yy21;
    }
yy64:
    ++YYCURSOR;
#line 403 "../src/main.ypp"
    {
        NEXT_ARG("--main-context-concurrency-hint",
                 opt_main_context_concurrency_hint);
    }
#line 766 "emilua.p/main.cpp"
yy65:
    ++YYCURSOR;
#line 399 "../src/main.ypp"
    {
        *cur_arg = YYCURSOR;
        goto opt_main_context_concurrency_hint;
    }
#line 774 "emilua.p/main.cpp"
}
#line 411 "../src/main.ypp"


opt_main_context_concurrency_hint:

#line 781 "emilua.p/main.cpp"
{
    char yych;
    yych = *YYCURSOR;
    switch (yych) {
        case '1': goto yy69;
        case 's': goto yy70;
        default: goto yy67;
    }
yy67:
    ++YYCURSOR;
yy68:
#line 415 "../src/main.ypp"
    { ERRARG("--main-context-concurrency-hint"); }
#line 795 "emilua.p/main.cpp"
yy69:
    yych = *++YYCURSOR;
    if (yych <= 0x00) goto yy71;
    goto yy68;
yy70:
    yych = *(YYMARKER = ++YYCURSOR);
    switch (yych) {
        case 'a': goto yy72;
        default: goto yy68;
    }
yy71:
    ++YYCURSOR;
#line 416 "../src/main.ypp"
    {
        main_ctx_concurrency_hint = 1;
        goto opt;
    }
#line 813 "emilua.p/main.cpp"
yy72:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'f': goto yy74;
        default: goto yy73;
    }
yy73:
    YYCURSOR = YYMARKER;
    goto yy68;
yy74:
    yych = *++YYCURSOR;
    switch (yych) {
        case 'e': goto yy75;
        default: goto yy73;
    }
yy75:
    yych = *++YYCURSOR;
    if (yych >= 0x01) goto yy73;
    ++YYCURSOR;
#line 420 "../src/main.ypp"
    {
#if EMILUA_CONFIG_USE_STANDALONE_ASIO
        main_ctx_concurrency_hint = ASIO_CONCURRENCY_HINT_SAFE;
#else // EMILUA_CONFIG_USE_STANDALONE_ASIO
        main_ctx_concurrency_hint = BOOST_ASIO_CONCURRENCY_HINT_SAFE;
#endif // EMILUA_CONFIG_USE_STANDALONE_ASIO
        goto opt;
    }
#line 842 "emilua.p/main.cpp"
}
#line 428 "../src/main.ypp"

end:

    if (filename.size() == 0) {
        try {
            boost::nowide::cerr << "missing filename\n";
        } catch (const std::ios_base::failure&) {}
        return 2;
    }

    if (appctx.app_args.size() == 0) {
        appctx.app_args.reserve(2);
        appctx.app_args.emplace_back(argv[0]);
        appctx.app_args.emplace_back(filename);
    }

    if (
        auto it = appctx.app_env.find("EMILUA_PATH") ;
        it != appctx.app_env.end()
    ) {
        for (std::string_view spec{it->second} ;;) {
            std::string_view::size_type sepidx = spec.find(
#if BOOST_OS_WINDOWS
                ';'
#else
                ':'
#endif
            );
            if (sepidx == std::string_view::npos) {
                appctx.emilua_path.emplace_back(
                    emilua::widen_on_windows(spec), fs::path::native_format);
                appctx.emilua_path.back().make_preferred();
                break;
            } else {
                appctx.emilua_path.emplace_back(
                    emilua::widen_on_windows(spec.substr(0, sepidx)),
                    fs::path::native_format);
                appctx.emilua_path.back().make_preferred();
                spec.remove_prefix(sepidx + 1);
            }
        }
    }

    appctx.emilua_path.emplace_back(
        emilua::widen_on_windows(EMILUA_CONFIG_LIBROOTDIR),
        fs::path::native_format);
    appctx.emilua_path.back().make_preferred();
    // TODO: Remove VERSION_MINOR from path components once emilua reaches
    // version 1.0.0 (versions that differ only in minor and patch numbers do
    // not break API).
    appctx.emilua_path.back() /=
        "emilua-" BOOST_PP_STRINGIZE(EMILUA_CONFIG_VERSION_MAJOR)
        "." BOOST_PP_STRINGIZE(EMILUA_CONFIG_VERSION_MINOR);

    {
#if EMILUA_CONFIG_THREAD_SUPPORT_LEVEL == 2
        asio::io_context ioctx{main_ctx_concurrency_hint};
        asio::make_service<emilua::properties_service>(
            ioctx, main_ctx_concurrency_hint);
#elif EMILUA_CONFIG_THREAD_SUPPORT_LEVEL == 1
        asio::io_context ioctx{1};
        asio::make_service<emilua::properties_service>(ioctx, 1);
#elif EMILUA_CONFIG_THREAD_SUPPORT_LEVEL == 0
# if EMILUA_CONFIG_USE_STANDALONE_ASIO
        asio::io_context ioctx{ASIO_CONCURRENCY_HINT_UNSAFE};
# else // EMILUA_CONFIG_USE_STANDALONE_ASIO
        asio::io_context ioctx{BOOST_ASIO_CONCURRENCY_HINT_UNSAFE};
# endif // EMILUA_CONFIG_USE_STANDALONE_ASIO
        asio::make_service<emilua::properties_service>(ioctx, 1);
#else
# error Invalid thread support level
#endif

        try {
            auto vm_ctx = emilua::make_vm(ioctx, appctx,
                                          main_context_type,
                                          emilua::widen_on_windows(filename));
            appctx.master_vm = vm_ctx;
            vm_ctx->strand().post([vm_ctx]() {
                vm_ctx->fiber_resume(
                    vm_ctx->L(),
                    hana::make_set(
                        emilua::vm_context::options::skip_clear_interrupter));
            }, std::allocator<void>{});
        } catch (std::exception& e) {
            try {
                boost::nowide::cerr << "Error starting the lua VM: " <<
                    e.what() << std::endl;
            } catch (const std::ios_base::failure&) {}
            return 1;
        }

        ioctx.run();
    }

    {
        std::unique_lock<std::mutex> lk{appctx.extra_threads_count_mtx};
        while (appctx.extra_threads_count > 0)
            appctx.extra_threads_count_empty_cond.wait(lk);
    }

    return appctx.exit_code;
}
